// This is vxl/VNL/nonlinear_minimizer.cxx
#ifdef _WIN32
#pragma warning(disable:4275) // disable C4275 warning
#endif

#ifdef VCL_NEEDS_PRAGMA_INTERFACE
#pragma implementation
#endif
/**
* \file
* \brief

* \author Andrew W. Fitzgibbon, Oxford RRG, 22 Aug 99
*
*/

#include <cstdio>   // sprintf()
#include <iostream>
#include "nonlinearminimizer.h"

/** Default ctor sets verbosity etc.
*/
VNL::NonlinearMinimizer::NonlinearMinimizer()
{
  xtol = 1e-8;           // Termination tolerance on X (solution vector)
  maxfev = 2000; // Termination maximum number of iterations.
  ftol = xtol * 0.01;    // Termination tolerance on F (sum of squared residuals)
  gtol = 1e-5;           // Termination tolerance on Grad(F)' * F = 0
  epsfcn = xtol * 0.001; // Step length for FD Jacobian
  trace = false;
  verbose_ = false;
  check_derivatives_=0;
  _Reset();
}

VNL::NonlinearMinimizer::~NonlinearMinimizer()
{
}

VNL::Matrix<double> const& VNL::NonlinearMinimizer::GetCovariance()
{
  static VNL::Matrix<double> null;
  return null;
}

void VNL::NonlinearMinimizer::_Reset()
{
  num_iterations_ = 0;
  num_evaluations_ = 0;
  start_error_ = 0;
}

/** Called by derived classes after each function evaluation.
*/
void VNL::NonlinearMinimizer::_ReportEval(double f)
{
  if (num_evaluations_ == 0) {
    start_error_ = f;
    end_error_ = f;
  }
  if (f < end_error_)
    end_error_ = f;

  ++num_evaluations_;
}

/** Called by derived classes after each iteration.
*/
void VNL::NonlinearMinimizer::_ReportIter()
{
  ++num_iterations_;
  if (verbose_) {
    char buf[1024];
    sprintf(buf, "Iter %4d, Eval %4d: Best F = %10g\n",
                num_iterations_, num_evaluations_, end_error_);
    std::cerr << buf;
  }
}

/** Return the name of the class.
*  Used by polymorphic IO
*/
std::string VNL::NonlinearMinimizer::IsA() const
{
  static const std::string class_name_="nonlinear_minimizer";
  return class_name_;
}

/** Return true if the name of the class matches the argument.
*  Used by polymorphic IO
*/
bool VNL::NonlinearMinimizer::IsClass(std::string const& s) const
{
  return s==IsA();
}

